@@ -1,7 +1,7 @@
 package Rope;
 
 use 5.006; use strict; use warnings;
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 use Rope::Object;
 my (%META, %PRO);
 our @ISA;
@@ -16,11 +16,33 @@ BEGIN {
 		scope => sub {
 			my ($caller, $self, %props) = @_;
 			for my $prop (keys %{$props{properties}}) {
-				$props{properties}->{$prop} = {%{$props{properties}{$prop}}};
+				#$props{properties}->{$prop} = {%{$props{properties}{$prop}}};
 				if ($props{properties}{$prop}{value} && ref $props{properties}{$prop}{value} eq 'CODE') {
 					my $cb = $props{properties}{$prop}{value};
 					$props{properties}{$prop}{value} = sub { $cb->($META{initialised}{$caller}{${$self}->{identifier}}, @_) };
 				}
+				for (qw/predicate clearer/) {	
+					if ($props{properties}->{$prop}->{$_}) {
+						my $prep = $_ eq 'predicate' ? 'has_' : 'clear_';
+						my $pred = $props{properties}->{$prop}->{$_};
+						my $ref = ref($pred);
+						if ( !$ref ) {
+							$props{properties}->{$pred =~ m/^\d+$/ ? "$prep$prop" : $pred} = {
+								value => $_ eq 'predicate' 
+									? sub { return defined $META{initialised}{$caller}{${$self}->{identifier}}->{$prop} ? 1 : 0 }
+									: sub { $META{initialised}{$caller}{${$self}->{identifier}}->{$prop} = undef }
+							};
+						} elsif ($ref eq 'CODE') {
+							$props{properties}->{"$prep$prop"} = {
+								value => sub { $pred->($META{initialised}{$caller}{${$self}->{identifier}}, $prop) }
+							};
+						} elsif ($ref eq 'HASH') {
+							my $cb = $pred->{value};
+							$pred->{value} = sub { $cb->($META{initialised}{$caller}{${$self}->{identifier}}, $prop) };
+							$props{properties}->{$pred->{name} || "$prep$prop"} = $pred;
+						}
+					}
+				}
 				for (qw/trigger delete_trigger/) {
 					my $trigger = $props{properties}{$prop}{$_};
 					if (defined $trigger) {
@@ -80,7 +102,7 @@ BEGIN {
 		},
 		set_prop => sub {
 			my ($caller, $prop, %options) = @_;
-			if ($META{$caller}{properties}{$prop}) {
+			if (exists $META{$caller}{properties}{$prop}) {
 				defined $options{$_} && do { $META{$caller}{properties}{$prop}{$_} = $options{$_} } for qw/builder trigger delete_trigger/;
 				if ($META{$caller}{properties}{$prop}{writeable}) {
 					$META{$caller}{properties}{$prop}{value} = $options{value} if exists $options{value};
@@ -97,7 +119,7 @@ BEGIN {
 				}
 			} else {
 				$META{$caller}{properties}{$prop} = {
-					class => $caller,
+					class => $options{class} || $caller,
 					index => ++$META{$caller}{keys},	
 					%options
 				};
@@ -132,10 +154,9 @@ BEGIN {
 				my (@properties) = @_;
 				while (@properties) {
 					my ($prop, $options) = (shift @properties, shift @properties);
-
 					my $ref = ref $options;					
 					if (!$ref || $ref ne 'HASH' || ! grep { defined $options->{$_} } 
-						qw/initable writeable builder enumerable configurable trigger delete_trigger value/
+						qw/initable writeable builder enumerable configurable trigger clearer predicate delete_trigger value/
 					) {
 						$options = {
 							initable => 1,
@@ -416,11 +437,8 @@ sub new {
 		eval $c;
 	}
 
-	$META{$name}{locked} = $meta->{locked} if (defined $meta->{locked});
-	$PRO{requires}($name)(@{$meta->{requires}}) if ($meta->{requires});
-	$PRO{extends}($name)(@{$meta->{extends}}) if ($meta->{extends});
-	$PRO{with}($name)(@{$meta->{with}}) if ($meta->{with});
-	$PRO{properties}($name)(ref $meta->{properties} eq 'ARRAY' ? @{$meta->{properties}} : %{$meta->{properties}}) if ($meta->{properties});
+	$pkg->set_meta($meta, $name);
+
 	return $PRO{new}($name)($name, %params);
 }
 
@@ -434,6 +452,27 @@ sub get_meta {
 	return $PRO{clone}($META{$caller || ref $self});
 }
 
+sub set_meta {
+	my ($self, $meta, $name) = @_;
+	$name = $meta->{name} if ! $name;
+	$META{$name}{locked} = $meta->{locked} if (defined $meta->{locked});
+	$PRO{requires}($name)(ref $meta->{requires} eq 'ARRAY' ? @{$meta->{requires}} : keys %{$meta->{requires}}) if ($meta->{requires});
+	$PRO{extends}($name)(@{$meta->{extends}}) if ($meta->{extends});
+	$PRO{with}($name)(@{$meta->{with}}) if ($meta->{with});
+	$PRO{properties}($name)(ref $meta->{properties} eq 'ARRAY' ? @{$meta->{properties}} : %{$meta->{properties}}) if ($meta->{properties})
+}
+
+sub clear_meta {
+	my ($self, $name) = @_;
+	$META{$name} = {
+		name => $name,
+		locked => 0,
+		properties => {},
+		requires => {},
+		keys => 0
+	};
+}
+
 1;
 
 __END__
@@ -444,7 +483,7 @@ Rope - Tied objects
 
 =head1 VERSION
 
-Version 0.19
+Version 0.20
 
 =cut
 
@@ -546,6 +585,14 @@ The trigger property/key expects a code ref that expects the value for that prop
 
 A delete trigger works the same as a trigger but is called on delete instead of set. The second param is the last set value for that property.
 
+=head2 predicate
+
+A predicate allows you to define an accessor to check whether a property is set. You have a few options, you can pass a positive integer or code reference and that will default the predicate to "has_$prop". You can also pass a string to name the accessor like "has_my_thing" or finally if you would like more customisation  you can pass a hash that defines the object properties. You cannot create triggers or hooks on predicates directly.
+
+=head2 clearer
+
+A clearer allows you to define an accessor to clear a property (set it to undef).  You have a few options, you can pass a positive integer or code reference and that will default the clearer to "clear_$prop". You can also pass a string to name the accessor like "clear_my_thing" or finally if you would like more customisation you can pass a hash that defines the object properties. You cannot create triggers or hooks on clearers. when using clearers and type checking in conjuction you need to ensure the type supports undef or you need to create your own custom clearer using a code ref or hash ref.
+
 =head2 index
 
 The index property/key expects an integer, if you do not set then this integer it's automatically generated and associated to the property. You will only want to set this is you always want to have a property last when itterating
@@ -791,6 +838,42 @@ This returns the existing META definition for an object.
 
 NOTE: this is now read only
 
+=head2 clear_meta
+
+	Rope->clear_meta('Knot');
+
+=head2 set_meta
+
+Extend or redefine an object meta definition.
+
+	package Knot;
+
+	use Rope;
+
+	prototyped (
+		one => 1,
+		two => 2,
+		three =>3
+	);
+
+	1;
+
+	package Hitch
+
+	extends 'Knot';
+
+	1;
+
+	my $meta = Ropt->get_meta('Hitch');
+
+	Rope->clear_meta('Hitch');
+
+	delete $meta->{properties};
+
+	Rope->set_meta($meta);
+
+	Hitch->new->one;
+
 =head1 OBJECT CLASS DEFINITION
 
 	package Builder;
@@ -860,6 +943,23 @@ Instead of
 
 	$builder->{thing} = 10;
 
+=head2 MONKEY PATCHING
+
+If for some reason you do not like the Autoload approach then Rope also has support for monkey patching the package routines.
+
+	package Builder;
+
+	use Rope;
+	use Rope::Monkey;
+
+	...
+
+	monkey;
+
+	1;
+
+	$builder->thing = 10;
+
 =head1 TYPES
 
 Rope also includes additional helpers for defining properties with fixed types, see C<Rope::Type> for more information. ( internally that uses C<Type::Standard> for the actual type checking. )
