@@ -10,6 +10,36 @@ BEGIN {
 			my ($caller, $method, $cb) = @_;
 			no strict 'refs';
 			*{"${caller}::${method}"} = $cb;
+		},
+		monkey_patch => sub {
+			my ($caller, $meta, $build) = @_;
+			for my $key (keys %{$meta}) {
+				$caller->can($key) || $PRO{keyword}($caller, $key, $PRO{monkey_patch_sub}($key));
+				if ($build) {
+					for (qw/predicate clearer/) {
+						if ($meta->{$key}->{$_}) {
+							my $prep = $_ eq 'predicate' ? 'has_' : 'clear_';
+							my $pred = $meta->{$key}->{$_};
+							my $ref = ref($pred);
+							$pred = !$ref && $pred !~ m/^\d+$/
+								? $pred
+								: $ref eq 'HASH' && $pred->{name}
+									? $pred->{name}
+									: "$prep$key";
+							$caller->can($pred) || $PRO{keyword}($caller, $pred, $PRO{monkey_patch_sub}($pred));
+						}
+					}
+				}
+			}
+		},
+		monkey_patch_sub => sub {
+			my $key = shift;
+			return sub :lvalue {
+				my ($self) = shift;
+				return $self->{$key}(@_) if !want(qw'LVALUE ASSIGN') && ref $self->{$key} eq 'CODE'; 
+				$self->{$key} = $_[0] if defined $_[0];
+				return $self->{$key};
+			};
 		}
 	);
 }
@@ -22,27 +52,15 @@ sub import {
 			while (@_) {
 				my ($prop, $options) = (shift @_, shift @_);
 				$self->{$prop} = $options;
-				monkey_patch($caller, { $prop => 1 });
+				$PRO{monkey_patch}($caller, { $prop => 1 });
 			}
 		} else {
 			my $meta = Rope->get_meta($caller);
-			monkey_patch($caller, $meta->{properties});
+			$PRO{monkey_patch}($caller, $meta->{properties}, 1);
 		}
 	});
 }
 
-sub monkey_patch {
-	my ($caller, $meta) = @_;
-	for my $key (keys %{$meta}) {
-		$PRO{keyword}($caller, $key, sub :lvalue {
-			my ($self) = shift;
-			return $self->{$key}(@_) if !want(qw'LVALUE ASSIGN') && ref $self->{$key} eq 'CODE'; 
-			$self->{$key} = $_[0] if defined $_[0];
-			return $self->{$key};
-		});
-	}
-}
-
 1;
 
 =head1 NAME
@@ -51,7 +69,7 @@ Rope::Monkey - Rope Monkey Patching
 
 =head1 VERSION
 
-Version 0.19
+Version 0.20
 
 =cut
 
