@@ -41,34 +41,32 @@ sub compile {
 
 sub set_value {
 	my ($self, $key, $value, $spec) = @_;
-	if (defined $value) {
-		if ($spec->{trigger}) {
-			$value = $spec->{trigger}->($value);
-		}
-		if (ref($value || "") ne 'CODE') {
-			for (qw/before around after/) {
-				if ($spec->{$_}) {
-					my $val = $spec->{$_}->($value);
-					if (defined $val) {
-						$value = $val;
-					}
-				}
-			}
-		}
-		if ($spec->{type}) {
-			$value = eval {
-				 $spec->{type}->($value);
-			};
-			if ($@ || ! defined $value) {
-				my @caller = caller(1);
-				if ($caller[0] eq 'Rope::Object') {
-					die sprintf("Failed to instantiate object (%s) property (%s) failed type validation. %s", $self->{name}, $key, $@);
-				}
-				die sprintf("Cannot set property (%s) in object (%s) failed type validation on line %s file %s: %s", $key, $self->{name}, $caller[2], $caller[1], $@);
-			}
-		}
-		$spec->{value} = $value;
+	if ($spec->{trigger}) {
+		$value = $spec->{trigger}->($value);
 	}
+	if (ref($value || "") ne 'CODE') {
+		for (qw/before around after/) {
+			if ($spec->{$_}) {
+				my $val = $spec->{$_}->($value);
+				if (defined $val) {
+					$value = $val;
+				}
+			}
+		}
+	}
+	if ($spec->{type}) {
+		$value = eval {
+			 $spec->{type}->($value);
+		};
+		if ($@) {
+			my @caller = caller(1);
+			if ($caller[0] eq 'Rope::Object') {
+				die sprintf("Failed to instantiate object (%s) property (%s) failed type validation. %s", $self->{name}, $key, $@);
+			}
+			die sprintf("Cannot set property (%s) in object (%s) failed type validation on line %s file %s: %s", $key, $self->{name}, $caller[2], $caller[1], $@);
+		}
+	}
+	$spec->{value} = $value;
 	if ($spec->{required} && ! defined $spec->{value}) {
 		die sprintf "Required property (%s) in object (%s) not set", $key, $self->{name};
 	}
